"""Initialization file for the `click_extended.decorators.math` module."""

from click_extended.decorators.math.absolute import absolute
from click_extended.decorators.math.add import add
from click_extended.decorators.math.ceil import ceil
from click_extended.decorators.math.clamp import clamp
from click_extended.decorators.math.divide import divide
from click_extended.decorators.math.floor import floor
from click_extended.decorators.math.maximum import maximum
from click_extended.decorators.math.minimum import minimum
from click_extended.decorators.math.modulo import modulo
from click_extended.decorators.math.multiply import multiply
from click_extended.decorators.math.normalize import normalize
from click_extended.decorators.math.power import power
from click_extended.decorators.math.rounded import rounded
from click_extended.decorators.math.sqrt import sqrt
from click_extended.decorators.math.subtract import subtract
from click_extended.decorators.math.to_percent import to_percent

__all__ = [
    "absolute",
    "add",
    "ceil",
    "clamp",
    "divide",
    "floor",
    "maximum",
    "minimum",
    "modulo",
    "multiply",
    "normalize",
    "power",
    "rounded",
    "sqrt",
    "subtract",
    "to_percent",
]
